#include <ctype.h>
#include "SDL/SDL.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_ttf.h"
#include "brikx.h"

SDL_Surface *LoadImage(char *fname,int Rkey,int Gkey,int Bkey)
{
    SDL_Surface *loaded,*optimized=NULL;
    if (loaded=IMG_Load(fname))
      {
        if (optimized=SDL_DisplayFormat(loaded))
            SDL_SetColorKey(optimized,SDL_RLEACCEL | SDL_SRCCOLORKEY,SDL_MapRGB(optimized->format,Rkey,Gkey,Bkey));
        else return loaded;
        SDL_FreeSurface(loaded);
      }
    else return NULL;
    return optimized;
}

void PutSurface(SDL_Surface *src,SDL_Surface *dest,SDL_Rect pos,int clipx,int clipy,int width,int height)
{
     if (!src || !dest) return;
     SDL_Rect clip;
     clip.x=clipx;
     clip.y=clipy;
     clip.w=width;
     clip.h=height;
     SDL_BlitSurface(src,clipx==-1?NULL:&clip,dest,&pos);
}

void PutSurface(SDL_Surface *src,SDL_Surface *dest,int x,int y,int clipx,int clipy,int width,int height)
{
     SDL_Rect pos;
     pos.x=x;
     pos.y=y;
     PutSurface(src,dest,pos,clipx,clipy,width,height);
}

int LoadLevel(char *fname,level &t)
{
      FILE *f=fopen(fname,"r");
      if (!f) return 0;
      char s[140];
      int i,j,timelimit;
      fgets(s,130,f);
      timelimit=atoi(s);
      for (i=0; i<=7 && !feof(f); i++)
        {
          fgets(s,130,f);
          for (j=0; j<=7; j++)
            t[j][i]=(s[j*2]-'0')*10+s[j*2+1]-'0';
        }
      fclose(f);
      return timelimit;
}

int SaveLevel(char *fname,level t,int timelimit)
{
    FILE *f=fopen(fname,"w+");
    if (!f) return 0;
    int i,j;
    fprintf(f,"%d\n",timelimit);
    for (i=0; i<=7; i++)
      {
        for (j=0; j<=7; j++)
          fprintf(f,"%02d",t[j][i]);
        fprintf(f,"\n");
      }
    fclose(f);
    return 1;
}

int CheckMatches(level &table,level drop)
{
    level temp;
    int i,j,count=0;
    for (i=0; i<=7; i++)
      for (j=0; j<=7; j++)
        if (IsTile(table[j][i]) && !drop[j][i] &&
           ((!drop[j-1][i] && j && table[j-1][i]==table[j][i]) ||
           (!drop[j][i-1] && i && table[j][i-1]==table[j][i]) ||
           (!drop[j+1][i] && j<7 && table[j+1][i]==table[j][i]) ||
           (!drop[j][i+1] && i<7 && table[j][i+1]==table[j][i])))
            temp[j][i]=TILE_EMPTY;
        else temp[j][i]=table[j][i];
    for (i=0; i<=7; i++)
      for (j=0; j<=7; j++)
        if (IsTile(table[j][i]=temp[j][i])) count++;
    return count;
}

int Input(SDL_Surface *screen,TTF_Font *font,SDL_Color color,int x,int y,char *s,int maxlen)
{
    SDL_Surface *text=NULL;
    SDL_Event event; 
    bool end=false;
    char t[maxlen+1],ss[strlen(s)+maxlen+2];
    *t=0;
    while (!end)
      {
        sprintf(ss,"%s%s.",s,t);
        if (text) SDL_FreeSurface(text);
        text=TTF_RenderText_Solid(font,ss,color);
        PutSurface(text,screen,x,y);
        SDL_Flip(screen);
        while (SDL_PollEvent(&event))
          if (event.type==SDL_KEYDOWN)
	     switch (event.key.keysym.sym)
	       {
		 case SDLK_BACKSPACE:
		        if (*t) t[strlen(t)-1]=0;
		      break;
		 case SDLK_ESCAPE:
			*t=0;
		 case SDLK_RETURN:
			end=true;
		      break;
		 default:
		      if (isalnum(event.key.keysym.sym) && strlen(t)<maxlen) sprintf(t,"%s%c",t,event.key.keysym.sym);
	       }
}
    SDL_FreeSurface(text);
    strcpy(s,t);
    return strlen(t);
}

int SaveHigh(long score,char *name)
{
    int i,j;
    high h;
    char s[NAME_MAXLEN+2];
    FILE *f=fopen(FILE_HIGH,"r");
    if (f)
      {
        for (i=0; !feof(f); i++)
          {
            fgets(s,NAME_MAXLEN,f);
            h[i].score=atoi(s);
            fgets(h[i].name,NAME_MAXLEN,f);
            h[i].name[strlen(h[i].name)-1]=s[strlen(s)-1]=0;
          }
        fclose(f);
        while (i<MAX_HIGHENTRIES)
          {
            h[i].score=h[i].name[0]=0;
            i++;
          }
      }
    else
      for (i=0; i<MAX_HIGHENTRIES; i++)
        h[i].score=h[i].name[0]=0;
    for (i=0; i<MAX_HIGHENTRIES && score<h[i].score; i++)
      ;
    if (i<MAX_HIGHENTRIES)
      {
        for (j=MAX_HIGHENTRIES-1; j>i; j--)
          {
            h[j].score=h[j-1].score;
            strcpy(h[j].name,h[j-1].name);
          }
        h[i].score=score;
        strcpy(h[i].name,name);
        if (!(f=fopen(FILE_HIGH,"w+"))) return -1;
        for (j=0; j<MAX_HIGHENTRIES; j++)
          fprintf(f,"%d\n%s\n",h[j].score,h[j].name);
        fclose(f);
      }
    return i;
}
