#include <math.h>
#include "SDL/SDL.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_ttf.h"
#include "brikx.h"
#include "button.h"

char tileset [256];

bool Escape(SDL_Surface *screen,long &timer)
{
     Button *yes=new Button(270,200,100,40,IMAGE_YES),
             *no=new Button(270,280,100,40,IMAGE_NO);
     SDL_Surface *text=LoadImage(IMAGE_ASKQUIT);
     SDL_Event event;
     long t0=SDL_GetTicks();
     int quit=0;
     while (!quit)
       {
         PutSurface(text,screen,270,120);
         yes->Show(screen,yes->Inside(event)*127+128);
         no->Show(screen,no->Inside(event)*127+128);
         SDL_Flip(screen);
         while (SDL_PollEvent(&event))
           switch (event.type)
             {
               case SDL_MOUSEBUTTONDOWN:
                      if (yes->Inside(event)) quit=2;
                      if (no->Inside(event)) quit=1;
                    break;
               case SDL_KEYDOWN:
                      switch (event.key.keysym.sym)
                         {
                          case SDLK_ESCAPE:
                          case SDLK_n:
                                 quit=1;
                               break;
                          case SDLK_y:
                                 quit=2;
                               break;
                         }
                   break;
             }
       }
     SDL_FreeSurface(text);
     delete yes;
     delete no;
     timer+=SDL_GetTicks()-t0;
     return quit==2;
}

void Play(SDL_Surface *screen)
{
    bool quit,endgame=false;
    float time=0;
    int i,j,levnum=1,timelimit,mode=MODE_NORMAL;
    long timer,nulltime,step,paused,score=0;
    char s[1024];
    Button *button_build=new Button(500,260,60,60,IMAGE_BUILD),*button_clear=new Button(560,260,60,60,IMAGE_CLEAR),
	   *button_erase=new Button(500,330,60,60,IMAGE_ERASE),*button_normal=new Button(500,400,60,60,IMAGE_NORMAL),
	   *button_reload=new Button(560,330,60,60,IMAGE_RELOAD);
    level table,drop;
    SDL_Color color={200,255,220};
    SDL_Event event;
    TTF_Font *font=TTF_OpenFont(FONTNAME_SCORE,FONTSIZE_SCORE),*smallfont=TTF_OpenFont(FONTNAME_SCORE,FONTSIZE_SMALL);
    SDL_Surface *base=LoadImage(BACKGROUND_BASE),*tile[MAX_TILES],
                *light[3]={LoadImage(IMAGE_LIGHT_GREEN),LoadImage(IMAGE_LIGHT_YELLOW),LoadImage(IMAGE_LIGHT_RED)},
                *text=NULL;
    if (!font || !smallfont)
      {
        fprintf(stderr,"ERROR: Font file %s not found\n",FONTNAME_SCORE);
        return;
      }
    if (!base)
      {
        fprintf(stderr,"ERROR: Image file %s not found\n",BACKGROUND_BASE);
        return;
      }
    for (i=0; i<=2; i++)
      if (!(light[i]))
	  {
	    fprintf(stderr,"ERROR: Light %d image file not found\n",i);
	    return;
	  }
    for (i=0; i<MAX_TILES; i++)
      {
	    sprintf(s,"%s%s/%d.png",DIR_TILESET,tileset,i);
	    if (!(tile[i]=LoadImage(s))) fprintf(stderr,"WARNING: %s%s/%d.png not found!\n",DIR_TILESET,tileset,i);
      }
   while (!endgame)
   {
    quit=false;
    sprintf(s,"%s%d.lvl",DIR_LEVEL,levnum);
    for (i=0; i<=7; i++)
      for (j=0; j<=7; j++)
        drop[j][i]=0;
    SDL_FillRect(screen,NULL,0);
    timer=nulltime=SDL_GetTicks();
    paused=0;
    if (time>1 || !(timelimit=LoadLevel(s,table)))
      {
	    fprintf(stderr,"ERROR: Level file %s not found!\n",s);
	    endgame=true;
	  if (score)
	  {
	    SDL_FillRect(screen,NULL,0);
	    sprintf(s,"End score: %ld",score);
	    text=TTF_RenderText_Solid(font,s,color);
	    PutSurface(text,screen,(WIDTH-text->w)/2,100);
	    strcpy(s,"Enter your name:");
	    Input(screen,font,color,10,100+2*FONTSIZE_SCORE,s,NAME_MAXLEN);
	    SaveHigh(score,s);
	  }
      }
    else
    {
     while (!quit)
       {
         SDL_Delay(DELAY);
         sprintf(s,"%ld",score);
         if (text) SDL_FreeSurface(text);
         text=TTF_RenderText_Solid(font,s,color);
         for (i=0; i<=6; i++)
           for (j=0; j<=7; j++)
             if (IsTile(table[j][i]) && IsEmpty(table[j][i+1]) && !drop[j][i])
                 drop[j][i]=GRAVITY;
	 PutSurface(base,screen);
	 PutSurface(text,screen,618-text->w,190);
	 button_build->Show(screen,mode==MODE_BUILD?255:score>=cost[MODE_BUILD]?ALPHA2:ALPHA1);
	 button_clear->Show(screen,mode==MODE_CLEAR?255:score>=cost[MODE_CLEAR]?ALPHA2:ALPHA1);
	 button_erase->Show(screen,mode==MODE_ERASE?255:score>=cost[MODE_ERASE]?ALPHA2:ALPHA1);
	 button_reload->Show(screen,mode==MODE_RELOAD?255:score>=cost[MODE_RELOAD]?ALPHA2:ALPHA1);
	 button_normal->Show(screen,mode==MODE_NORMAL?255:ALPHA2);
	 sprintf(s,"%d",cost[MODE_BUILD]);
         if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,mode==MODE_BUILD?255:score>=cost[MODE_BUILD]?ALPHA2:ALPHA1);
	 PutSurface(text,screen,515,308);
	 sprintf(s,"%d",cost[MODE_CLEAR]);
         if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,mode==MODE_CLEAR?255:score>=cost[MODE_CLEAR]?ALPHA2:ALPHA1);
	 PutSurface(text,screen,575,308);
	 sprintf(s,"%d",cost[MODE_ERASE]);
         if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,mode==MODE_ERASE?255:score>=cost[MODE_ERASE]?ALPHA2:ALPHA1);
	 PutSurface(text,screen,510,378);
	 sprintf(s,"%d",cost[MODE_RELOAD]);
         if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,mode==MODE_RELOAD?255:score>=cost[MODE_RELOAD]?ALPHA2:ALPHA1);
	 PutSurface(text,screen,575,378);
	 sprintf(s,"%d",levnum);
         if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(font,s,color);
	 PutSurface(text,screen,618-text->w,20);
         step=SDL_GetTicks()-timer;
         timer=SDL_GetTicks();
         time=(timer-nulltime-paused)/1000.0/timelimit;
          for (i=0; i<=2; i++)
	   {
             if (time<0.75)
               {
                 if (time>(i+1)/4.0) j=255;
                 else if (time<i/4.0) j=0;
                 else j=round((time-i/4.0)*1000);
               }
             else j=timer%500>250?0:255;
             SDL_SetAlpha(light[i],SDL_SRCALPHA | SDL_RLEACCEL,j);
             PutSurface(light[i],screen,500,120-i*50);
           }
         quit=!CheckMatches(table,drop) || time>1;
	 for (i=0; i<=7; i++)
	   for (j=0; j<=7; j++)
	       {
                  PutSurface(tile[table[j][i]],screen,j*TILE_WIDTH+WINDOWX,round((i+(drop[j][i]==0?0:1)-drop[j][i]/float(GRAVITY))*TILE_HEIGHT+WINDOWY));
                  if (drop[j][i])
                    if ((drop[j][i]-=step)<=0)
                      {
                        drop[j][i]=0;
                        table[j][i+1]=table[j][i];
                        table[j][i]=TILE_EMPTY;
                      }
               }
	 SDL_Flip(screen);
	 while (SDL_PollEvent(&event))
	 {
	   SDL_Delay(1);
	   switch (event.type)
		     {
		       case SDL_QUIT:
			          quit=endgame=Escape(screen,paused);
			        break;
		       case SDL_KEYDOWN:
			      switch (event.key.keysym.sym)
				{
				  case ' ':
				         mode=MODE_NORMAL;
				       break;
				  case SDLK_TAB:
				         while (score<cost[++mode])
					    ;
					 if (mode==MODE_END) mode=0;
				       break;
				  case SDLK_p:
				       break;
				  case SDLK_ESCAPE:
				  case SDLK_q:
				  case SDLK_F10:
					     quit=endgame=Escape(screen,paused);
				       break;
				  case SDLK_F12:
				         if (SDL_GetModState() & KMOD_RCTRL)
					     for (i=0; i<=7; i++)
						 for (j=0; j<=7; j++)
						     table[j][i]=TILE_EMPTY;
				       break;
				}
			    break;
			case SDL_MOUSEBUTTONDOWN:
				j=(event.button.x-WINDOWX)/TILE_WIDTH;
				i=(event.button.y-WINDOWY)/TILE_HEIGHT;
			        if (button_build->Inside(event) && score>=cost[MODE_BUILD]) mode=MODE_BUILD;
				else if (button_clear->Inside(event) && score>=cost[MODE_CLEAR]) mode=MODE_CLEAR;
				else if (button_normal->Inside(event)) mode=MODE_NORMAL;
				else if (button_erase->Inside(event) && score>=cost[MODE_ERASE]) mode=MODE_ERASE;
				else if (button_reload->Inside(event) && score>=cost[MODE_RELOAD])
				  {
				    quit=true;
				    score-=cost[MODE_RELOAD];
				  }
				else if (event.button.button==SDL_BUTTON_MIDDLE) mode=MODE_NORMAL;
				else if (j<=7 && i<=7)
				  switch (mode)
				  {
				    case MODE_NORMAL:
					switch (event.button.button)
					  {
					    case SDL_BUTTON_LEFT:
						   if (IsTile(table[j][i]) && j && IsEmpty(table[j-1][i]))
						     {
						       table[j-1][i]=table[j][i];
						       table[j][i]=TILE_EMPTY;
						     }
						 break;
					    case SDL_BUTTON_RIGHT:
						   if (IsTile(table[j][i]) && j<7 && IsEmpty(table[j+1][i]))
						     {
						       table[j+1][i]=table[j][i];
						       table[j][i]=TILE_EMPTY;
						     }
						 break;
					  }
					break;
				    case MODE_BUILD:
					   if (IsEmpty(table[j][i]) && score>=cost[MODE_BUILD])
					     {
					       score-=cost[MODE_BUILD];
					       table[j][i]=TILE_WALL;
					     }
					break;
				    case MODE_CLEAR:
					   if (table[j][i]==TILE_WALL && score>=cost[MODE_CLEAR])
					     {
					       score-=cost[MODE_CLEAR];
					       table[j][i]=TILE_EMPTY;
					     }
					break;
				    case MODE_ERASE:
					   if (IsTile(table[j][i]) && score>=cost[MODE_CLEAR])
					     {
					       score-=cost[MODE_ERASE];
					       table[j][i]=TILE_EMPTY;
					     }
					break;
				  }
			     break;
		     }
	       }
	    }
    if (time<=1 && !CheckMatches(table,drop)) score+=round((1-time)*levnum*100)+levnum*10;
    if (!CheckMatches(table,drop)) levnum++;
    }
   }
    SDL_FreeSurface(base);
    SDL_FreeSurface(text);
    TTF_CloseFont(font);
    TTF_CloseFont(smallfont);
    delete button_build;
    delete button_clear;
    delete button_erase;
    delete button_normal;
    delete button_reload;
    for (i=0; i<3; i++) SDL_FreeSurface(light[i]);
    for (i=0; i<MAX_TILES; i++) if (tile[i]) SDL_FreeSurface(tile[i]);
}

void Options(SDL_Surface *screen)
{
}

void Edit(SDL_Surface *screen)
{
    bool quit=false;
    int i,j,levnum=1,timelimit,active=TILE_FIRST;
    char s[1024],t[1024];
    level table;
    SDL_Color color={200,255,220};
    SDL_Event event;
    Uint32 rmask,gmask,bmask,amask;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif    
    TTF_Font *font=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SCORE),*smallfont=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SMALL);
    SDL_Surface *base=LoadImage(BACKGROUND_EDIT),*tile[MAX_TILES],*tile_small[MAX_TILES],*text=NULL;
    Button *load=new Button(WINDOWX+TILE_WIDTH*8+TILE_WIDTH/2,WINDOWY+TILE_HEIGHT*7,32,32,IMAGE_LOAD),*save=new Button(WINDOWX+TILE_WIDTH*9+4,WINDOWY+TILE_HEIGHT*7+4,24,24,IMAGE_SAVE),
           *clock=new Button(WINDOWX+TILE_WIDTH*9+TILE_WIDTH/2,WINDOWY+TILE_HEIGHT*7,32,32,IMAGE_CLOCK);
    if (!font || !smallfont)
      {
        fprintf(stderr,"ERROR: Font file %s not found\n",FONTNAME_SCORE);
        return;
      }
    if (!base)
      {
        fprintf(stderr,"ERROR: Image file %s not found\n",BACKGROUND_BASE);
        return;
      }
    for (i=0; i<MAX_TILES; i++)
      {
	    sprintf(s,"%s%s/%d.png",DIR_TILESET,tileset,i);
	    if (!(tile[i]=LoadImage(s))) fprintf(stderr,"WARNING: %s%s/%d.png not found!\n",DIR_TILESET,tileset,i);
	    sprintf(s,"%s%s/small/%d.png",DIR_TILESET,tileset,i);
	    if (!(tile_small[i]=LoadImage(s))) fprintf(stderr,"WARNING: %s%s/small/%d.png not found!\n",DIR_TILESET,tileset,i);
      }
    SDL_FillRect(screen,NULL,1);
    sprintf(s,"%s%d.lvl",DIR_LEVEL,levnum);
    if ((timelimit=LoadLevel(s,table))>=0)
    while (!quit)
    {
        SDL_Delay(DELAY);
        PutSurface(base,screen);
	for (i=0; i<=3; i++)
	  for (j=0; j<=8; j++)
	    if (tile_small[TILE_FIRST+i*9+j])
	    {
	      SDL_SetAlpha(tile_small[TILE_FIRST+i*9+j],SDL_SRCALPHA | SDL_RLEACCEL,TILE_FIRST+i*9+j==active?255:ALPHA2);
	      PutSurface(tile_small[TILE_FIRST+i*9+j],screen,WINDOWX+TILE_WIDTH*8+TILE_WIDTH*(i+1)/2,WINDOWY+TILE_HEIGHT*(j+1)/2);
	    }
	for (i=0; i<=2; i++)
	  for (j=0; j<=3;j++)
	    if (tile_small[i*4+j])
	    {
	      SDL_SetAlpha(tile_small[i*3+j],SDL_SRCALPHA | SDL_RLEACCEL,i*3+j==active?255:ALPHA2);
	      PutSurface(tile_small[i*3+j],screen,WINDOWX+TILE_WIDTH*8+TILE_WIDTH*(i+1)/2,WINDOWY+TILE_HEIGHT*(j+10)/2);
	    }
	for (i=0; i<=7; i++)
	  for (j=0; j<=7; j++)
	      PutSurface(tile[table[j][i]],screen,j*TILE_WIDTH+WINDOWX,i*TILE_HEIGHT+WINDOWY);
	load->Show(screen);
	save->Show(screen);
	clock->Show(screen);
	SDL_Flip(screen);
        while (SDL_PollEvent(&event))
        {
          SDL_Delay(1);
          switch (event.type)
            {
              case SDL_QUIT:
                     quit=true;
                   break;
              case SDL_MOUSEBUTTONDOWN:
                     if (load->Inside(event))
                       {
                         sprintf(s,"Load level (current:%d):",levnum);
                         if (Input(screen,font,color,10,200,s,100) && atoi(s)>0)
                           {
				 levnum=atoi(s);
				 sprintf(t,"%s%s.lvl",DIR_LEVEL,s);
				 fprintf(stderr,"Loading: %s\n",t);
				 if (!(timelimit=LoadLevel(t,table)))
				   fprintf(stderr,"Error loading level %s",t);
                           }
                       }
                     else if (save->Inside(event)) 
                       {
                         sprintf(s,"Save as level number (current:%d):",levnum);
                         if (Input(screen,font,color,10,200,s,100) && atoi(s)>0)
                           {
                             levnum=atoi(s);
                             sprintf(t,"%s%s.lvl",DIR_LEVEL,s);
                             SaveLevel(t,table,timelimit);
                           }
                       }
                     else if (clock->Inside(event))
                       {
                         sprintf(s,"Enter timelimit (current:%d):",timelimit);
                         if (Input(screen,font,color,10,200,s,100) && atoi(s)>0) timelimit=atoi(s);
                       }
                     else 
                     if (event.button.x<TILE_WIDTH*8+WINDOWX)
                       {
                         j=(event.button.x-WINDOWX)/TILE_WIDTH;
		         i=(event.button.y-WINDOWY)/TILE_HEIGHT;
		         switch (event.button.button)
		           {
		             case SDL_BUTTON_LEFT:
		                    table[j][i]=active;
		                  break;
		             case SDL_BUTTON_RIGHT:
		                    table[j][i]=TILE_EMPTY;
		                  break;
		           }
		       }
		     else
		       {
                         j=(event.button.x-TILE_WIDTH*8-WINDOWX-TILE_WIDTH/2)/(TILE_WIDTH/2);
                         i=(event.button.y-WINDOWY-TILE_HEIGHT/2)/(TILE_HEIGHT/2);
                         if (i<=8 && tile[TILE_FIRST+j*9+i]) active=TILE_FIRST+j*9+i;
                         else if (tile[(i-9)+j*3]) active=(i-9)+j*3;
		       }
                   break;
              case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=true;
                              break;
                       }
                   break;
            }
          }
    }
    for (i=0; i<MAX_TILES; i++)
      {
        if (tile[i]) SDL_FreeSurface(tile[i]);
        if (tile_small[i]) SDL_FreeSurface(tile_small[i]);
      }
    TTF_CloseFont(font);
    delete load;
    delete save;
    delete clock;
    TTF_CloseFont(smallfont);
    SDL_FreeSurface(base);
    if (text) SDL_FreeSurface(text);
}

void Help(SDL_Surface *screen)
{
    SDL_Surface *help=NULL,*back=NULL;
    if (!(help=LoadImage(BACKGROUND_HELP))) return;
    bool quit=false;
    Uint32 rmask,gmask,bmask,amask;
    SDL_Event event;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif
    if (!(back=SDL_CreateRGBSurface(SDL_SRCALPHA | SDL_SWSURFACE | SDL_RLEACCEL,WIDTH-80,HEIGHT-80,BPP,rmask,gmask,bmask,amask))) return;
    SDL_FillRect(back,NULL,100*amask);
    PutSurface(back,screen,40,40);
    PutSurface(help,screen);
    SDL_Flip(screen);
    while (!quit)
      {
        SDL_Delay(DELAY);
        while (SDL_PollEvent(&event))
        {
          SDL_Delay(1);
          switch (event.type)
            {
              case SDL_QUIT:
                     quit=true;
                   break;
              case SDL_MOUSEBUTTONDOWN:
                     quit=true;
                   break;
              case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=true;
                              break;
                       }
                   break;
            }
          }
      }
    SDL_FreeSurface(back);
    SDL_FreeSurface(help);
}

void Highs(SDL_Surface *screen)
{
    TTF_Font *font=NULL;
    SDL_Surface *text=NULL,*back=NULL;
    SDL_Event event;
    SDL_Color color={100,200,255};
    font=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SMALL);
    if (!font) return;
    Uint32 rmask,gmask,bmask,amask;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif
    if (!(back=SDL_CreateRGBSurface(SDL_SRCALPHA | SDL_SWSURFACE | SDL_RLEACCEL,FONTSIZE_SMALL*(NAME_MAXLEN+7),MAX_HIGHENTRIES*FONTSIZE_SMALL,BPP,rmask,gmask,bmask,amask))) return;
    SDL_FillRect(back,NULL,100*amask);
    bool quit=false;
    int i;
    char s[NAME_MAXLEN+1],s2[NAME_MAXLEN+1];
    FILE *f=fopen(FILE_HIGH,"r");
    if (!f) return;
    for (i=0; !feof(f); i++)
      {
        fgets(s,NAME_MAXLEN,f);
        fgets(s2,NAME_MAXLEN,f);
        if (s[strlen(s)-1]=='\n') s[strlen(s)-1]=0;
        if (s2[strlen(s2)-1]=='\n') s2[strlen(s2)-1]=0;
	if (*s2)
	{
	    if (text) SDL_FreeSurface(text);
	    text=TTF_RenderText_Solid(font,s2,color);
	    PutSurface(text,back,0,i*FONTSIZE_SMALL);
	    if (text) SDL_FreeSurface(text);
	    text=TTF_RenderText_Solid(font,s,color);
	    PutSurface(text,back,back->w-text->w,i*FONTSIZE_SMALL);
	}
      }
    fclose(f);
    PutSurface(back,screen,(screen->w-back->w)/2,(screen->h-back->h)/2);
    SDL_Flip(screen);
    while (!quit)
      {
        SDL_Delay(DELAY);
        while (SDL_PollEvent(&event))
        {
          SDL_Delay(1);
          switch (event.type)
            {
              case SDL_QUIT:
                     quit=true;
                   break;
              case SDL_MOUSEBUTTONDOWN:
                     quit=true;
                   break;
              case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=true;
                              break;
                       }
                   break;
            }
          }
      }
    TTF_CloseFont(font);
    SDL_FreeSurface(text);
    SDL_FreeSurface(back);
}

int main(int argc,char *argv[])
{
    bool quit=false;
    SDL_Surface *menu=NULL,*screen=NULL,*logo=NULL,*text=NULL;
    SDL_Event   event;
    strcpy(tileset,TILESET_DEFAULT);
    if (SDL_Init(SDL_INIT_EVERYTHING)==-1) return -1;
    TTF_Font *font=NULL;
    TTF_Init();
    font=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SMALL);
    SDL_Color color={200,255,255};
    screen=SDL_SetVideoMode(WIDTH,HEIGHT,BPP,SDL_SWSURFACE);
    Button *button_play=new Button(450,50,158,40,IMAGE_PLAY),
           *button_edit=new Button(450,110,158,40,IMAGE_EDIT),
           *button_options=new Button(450,170,158,40,IMAGE_OPTIONS),
           *button_help=new Button(450,230,158,40,IMAGE_HELP),
           *button_highs=new Button(450,290,158,40,IMAGE_HIGHS),
           *button_quit=new Button(450,350,158,40,IMAGE_QUIT);
    if (!screen)
      {
        SDL_Quit();
        return -2;
      }
    SDL_WM_SetCaption("Brikx by glezmen",NULL);
    menu=LoadImage(BACKGROUND_MENU);
    logo=LoadImage(IMAGE_LOGO);
    SDL_SetAlpha(logo,SDL_SRCALPHA | SDL_RLEACCEL,192);
    if (font) text=TTF_RenderText_Solid(font,"Brikx by glezmen (bokorn@tvn.hu)     Version: "VERSION,color);
    while (!quit)
      {
        SDL_Delay(DELAY);
        SDL_FillRect(screen,NULL,0);
        PutSurface(menu,screen);
        PutSurface(logo,screen,80,40);
        button_quit->Show(screen,button_quit->Inside(event)?255:ALPHA2);
        button_options->Show(screen,button_options->Inside(event)?255:ALPHA2);
        button_play->Show(screen,button_play->Inside(event)?255:ALPHA2);
        button_help->Show(screen,button_help->Inside(event)?255:ALPHA2);
        button_highs->Show(screen,button_highs->Inside(event)?255:ALPHA2);
        button_edit->Show(screen,button_edit->Inside(event)?255:ALPHA2);
        if (text) PutSurface(text,screen,10,HEIGHT-FONTSIZE_SMALL*2);
        SDL_Flip(screen);
        while (SDL_PollEvent(&event))
          switch (event.type)
            {
              case SDL_QUIT:
                     quit=true;
                   break;
              case SDL_MOUSEBUTTONDOWN:
                     if (button_quit->Inside(event)) quit=true;
                     if (button_play->Inside(event)) Play(screen);
                     if (button_options->Inside(event)) Options(screen);
                     if (button_help->Inside(event)) Help(screen);
                     if (button_highs->Inside(event)) Highs(screen);
                     if (button_edit->Inside(event)) Edit(screen);
                   break;
              case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
                         case SDLK_o:
                                Options(screen);
                              break;
                         case SDLK_p:
                         case SDLK_RETURN:
                                Play(screen);
                              break;
                         case SDLK_h:
                                Help(screen);
                              break;
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=true;
                              break;
                         case SDLK_e:
                                Edit(screen);
                              break;
                       }
                   break;
            }
      } 
    delete button_quit;
    delete button_play;
    delete button_options;
    delete button_edit;
    delete button_highs;
    delete button_help;
    SDL_FreeSurface(menu);
    SDL_FreeSurface(text);
    SDL_FreeSurface(logo);
    TTF_CloseFont(font);
    TTF_Quit();
    SDL_Quit();
    return 0;
}
